<?php
header("Content-type: image/png");

/**
  * @file
  * Background Gradient Image generator
  *
  * This file create a background gradient image using the gd library. It will return
  * the image directly and save it on the server for the next time. It will not
  * generate the same image twice.
  *
  * I used small portions of code from two web sites:
  * @see http://www.hawkee.com/snippet/5022/
  * @see http://ca.php.net/imagecolorresolve
  *
  * @author David Giard (swe3tdave)
  * @author Michael Lustfield (MTecknology)
  *
  * @param start Starting Color
  * @param end End Color
  *
  * @return Generated or Cached Image
  *
  * @note
  * Use it as you would a normal image, like this:
  * <img src="gradient.php?start=796646&end=D3CAAA">
  * Or inside a css file:
  * background-image: url(gradient.php?start=796646&end=D3CAAA);
  */

$_udtgr_height = 520;
$_udtgr_width = 1;
$default_background = "background.png";
$default_start_color = '796646';
$default_end_color = 'D3CAAA';
$_udtgr_default_image_path = ''; // Default to script directory

/**
  * Changing those will change how the gradient curve is generated. So you can try
  * changing them and see what appends. Here are the default setting in case you
  * want to change back :
  *
  * $_udtgr_gradient_start = 65;
  * $_udtgr_gradient_curve = 1.1;
  * $_udtgr_gradient_sum = 800 / $_udtgr_gradient_curve;
  */
$_udtgr_gradient_start = 65;
$_udtgr_gradient_curve = 1.1;
$_udtgr_gradient_sum = 800 / $_udtgr_gradient_curve;

/**
 * You shoudn't modify anything behond this point, unless you know what you're doing.
 */

$_udtgr_start = $default_start_color;
$_udtgr_end = $default_end_color;

if ($_GET["start"]) {
  $_udtgr_start = $_GET["start"];
}
if ($_GET["end"]) {
  $_udtgr_end = $_GET["end"];
}

function _myexactcolor($png, $r, $g, $b) {
  $mycolor = imagecolorexact($png, $r, $g, $b);
  if ($mycolor == -1) {
    $mycolor = imagecolorallocate($png, $r, $g, $b);
  }
  if ($mycolor == -1) {
    $mycolor = imagecolorresolve($png, $r, $g, $b);
    #error_log("Damn!  STILL couldn't allocate the color!", 0);
  }
  return $mycolor;
}

function _createnewpng() {
  global $_udtgr_start, $_udtgr_end, $_udtgr_width, $_udtgr_height, $_udtgr_gradient_curve, $_udtgr_gradient_start, $_udtgr_gradient_sum, $_udtgr_default_image_path;

  if (!file_exists($_udtgr_default_image_path . $_udtgr_start . "-" . $_udtgr_end . ".png")) {
    $_udtgr_start_r = hexdec(drupal_substr($_udtgr_start, 0, 2));
    $_udtgr_start_g = hexdec(drupal_substr($_udtgr_start, 2, 2));
    $_udtgr_start_b = hexdec(drupal_substr($_udtgr_start, 4, 2));
    $_udtgr_end_r = hexdec(drupal_substr($_udtgr_end, 0, 2));
    $_udtgr_end_g = hexdec(drupal_substr($_udtgr_end, 2, 2));
    $_udtgr_end_b = hexdec(drupal_substr($_udtgr_end, 4, 2));
    $image = imagecreate($_udtgr_width, $_udtgr_height);

    $_udtgr_gradient_curve = $_udtgr_gradient_curve + ($_udtgr_start_r*(0.5/121));

    for ($i=0;$i<250;$i++) {
      $colorset[$i] = _myexactcolor($image, $_udtgr_start_r + ($i*(($_udtgr_end_r-$_udtgr_start_r)/250)), $_udtgr_start_g + ($i*(($_udtgr_end_g-$_udtgr_start_g)/250)), $_udtgr_start_b + ($i*(($_udtgr_end_b-$_udtgr_start_b)/250)));
    }
    for ($y=0;$y<$_udtgr_height;$y++) {
      for ($x=0;$x<$_udtgr_width;$x++) {

        if ($y<=$_udtgr_gradient_start) {
          imagesetpixel($image, $x, $y, $colorset[0] );
        }
        else {
          imagesetpixel($image, $x, $y, $colorset[(int)(((pow($y - $_udtgr_gradient_start, ($_udtgr_gradient_sum / ($y- $_udtgr_gradient_start))) * 250 * pow($_udtgr_height - $_udtgr_gradient_start, 0 - ($_udtgr_gradient_sum / ($y- $_udtgr_gradient_start)))) + (pow($y - $_udtgr_gradient_start, $_udtgr_gradient_curve)* 250 *pow($_udtgr_height - $_udtgr_gradient_start, 0-$_udtgr_gradient_curve)))/2)] );
        }
      }
    }
    imagepng($image, $_udtgr_default_image_path . $_udtgr_start . "-" . $_udtgr_end . ".png");
    imagepng($image);
    imagedestroy($image);
  }
  else {
    $h = fopen($_udtgr_default_image_path . $_udtgr_start . "-" . $_udtgr_end . ".png", "r");
    fpassthru($h);
    fclose($h);
  }
}


if ($_udtgr_start != $default_start_color || $_udtgr_end != $default_end_color) {
  _createnewpng();
}
else {
  if (file_exists($_udtgr_default_image_path . $default_background)) {
    $h = fopen($_udtgr_default_image_path . $default_background, "r");
    fpassthru($h);
    fclose($h);
  }
  else {
    _createnewpng();
  }
}
